//****************************************************************************
//Copyright (C) 2008 Airtight Interactive. All Rights Reserved.
//Usage of this code is subject to the Terms of Use accompanying
//this product.
//****************************************************************************

import mx.transitions.easing.Regular;
import mx.transitions.easing.Strong;
import mx.transitions.Tween;
import mx.utils.Delegate;

import com.airtightinteractive.apps.viewers.simpleViewer.ImageArea;
import com.airtightinteractive.apps.viewers.simpleViewer.ThumbArea;
import com.airtightinteractive.apps.viewers.simpleViewer.StageManager;
import com.airtightinteractive.apps.viewers.simpleViewer.Options;
import com.airtightinteractive.apps.viewers.simpleViewer.XMLManager;
import com.airtightinteractive.utils.RectUtil;
import com.airtightinteractive.utils.TweenExtended;

/**
 * Handles UI for a SimpleViewer Thumbnail. Most thumb elements are drawn programmatically 
 * via the Drawing API. Some Elements are attached from the library.
 * @author Felix Turner
 */
class com.airtightinteractive.apps.viewers.simpleViewer.Thumb {
	
	private static var sFrameTweenLen:Number = 10;
	
	private var mThumbArea:ThumbArea;
	private var mImageArea:ImageArea;
	private var mXMLManager:XMLManager;
	private var mStageManager:StageManager;

	private var mFrameWidth:Number;	
		
	private var mIndex:Number;	
	private var mThumbLoaded:Boolean;	
	private var mThumbURL:String;	
	private var mImageLoaded:Boolean;	
	private var mSelected:Boolean;	

	private var mTwnFade:Tween;
	private var mFrameTwn;//:Tween; - don't explicitly type to avoid compile errors in MX 04
	private var mPopupTween:TweenExtended;	

	private var mClip_mc:MovieClip;
	private var mBase_mc:MovieClip;
	private var mMask_mc:MovieClip;
	private var mFrame_mc:MovieClip;
	private var mLoader_mc:MovieClip;
	private var mImgLoadStatus_mc:MovieClip;
	private var mError_mc:MovieClip;
	private var mViewedIcon_mc:MovieClip;
	private var mInit_mc:MovieClip;
			
	function Thumb(target:MovieClip, index:Number, thumbURL:String){
								
		mThumbArea = ThumbArea.getInstance();	
		mImageArea = ImageArea.getInstance();
		mXMLManager = XMLManager.getInstance();	
		mStageManager = StageManager.getInstance();	
		mIndex = index;		
		mThumbURL = thumbURL;
		var d = target.getNextHighestDepth();		
		mClip_mc = target.createEmptyMovieClip("clip"+d,d);
		
		//Base MC
		mBase_mc = mClip_mc.createEmptyMovieClip("base",mClip_mc.getNextHighestDepth());			
		//Init MC preloader (little square)
		mInit_mc = mBase_mc.attachMovie("InitialIcon","mcInit",mBase_mc.getNextHighestDepth());
		mInit_mc._x = Math.round(Options.thumbnailWidth/2-mInit_mc._width/2);
		mInit_mc._y = Math.round(Options.thumbnailHeight/2-mInit_mc._height/2);
		var col = new Color(mInit_mc);
		col.setRGB(mXMLManager.thumbnailFrameColor);					
		//Loader MC
		mLoader_mc = mBase_mc.createEmptyMovieClip("loader",mBase_mc.getNextHighestDepth());				
		//Mask MC
		mMask_mc = mBase_mc.createEmptyMovieClip("mask",mBase_mc.getNextHighestDepth());			
		//ViewedIcon MC
		if (Options.showThumbnailVisited){
			mViewedIcon_mc = mBase_mc.attachMovie("DogEar","dogear",mBase_mc.getNextHighestDepth());
			var col2 = new Color(mViewedIcon_mc);
			col2.setRGB(mXMLManager.thumbnailFrameColor);
			//place in top-right corner
			mViewedIcon_mc._x = Options.thumbnailWidth - Options.thumbnailFrameWidth - 1 - mViewedIcon_mc._width;
			mViewedIcon_mc._y = Options.thumbnailFrameWidth + 1;
			mViewedIcon_mc._visible = false;
		}		
		//Frame MC
		mFrame_mc = mBase_mc.createEmptyMovieClip("frame",mBase_mc.getNextHighestDepth());	
		mFrame_mc._visible = false;		
		//ImgLoadStatus MC
		if (Options.showThumbnailLoadBar){
			mImgLoadStatus_mc = mBase_mc.createEmptyMovieClip("loadbar",mBase_mc.getNextHighestDepth());
		}	
														
		mFrameTwn = new Tween(this, "mFrameWidth", Strong.easeOut, Options.thumbnailFrameWidth, Options.thumbnailFrameWidth, sFrameTweenLen, false);		
   		mFrameTwn.onMotionChanged = Delegate.create(this, drawFrame);    		   		   		
		mSelected = false;
		mThumbLoaded = false;
		mImageLoaded = false;
		
		enableMouseActions();	
		
		if (mThumbArea.selectedThumbIndex == mIndex){
			select();
		}
													
	}
	
	function loadThumb(){		

		var loader_mcl:MovieClipLoader = new MovieClipLoader();
		loader_mcl.addListener(this);		
		loader_mcl.loadClip(mThumbURL,mLoader_mc);
	}
	
	
	/*
	 * Thumbnail image has loaded.
	 * 
	 */
	function onLoadInit(){
		mLoader_mc.forceSmoothing = Options.useSmoothing;
		mThumbLoaded = true;
				
		mInit_mc._visible = false;
		
		//crop thumb image 		
		if (mLoader_mc._width < mLoader_mc._height){
			mLoader_mc._width = Options.thumbnailWidth;
			mLoader_mc._yscale = mLoader_mc._xscale;		
		}else{
			mLoader_mc._height = Options.thumbnailHeight;
			mLoader_mc._xscale = mLoader_mc._yscale;
		}
		
		//center thumb image
		mLoader_mc._x = Math.floor((Options.thumbnailWidth-mLoader_mc._width)/2);
		mLoader_mc._y = Math.floor((Options.thumbnailHeight-mLoader_mc._height)/2);
				
		if (!mImageLoaded) mLoader_mc._alpha = Options.thumbnailLoadingAlpha;
						
		if (!mImageLoaded){
			showImageProgress(0);
		}	
		
		//draw mask
		mMask_mc.beginFill(0xffffff, 100);
		RectUtil.rectangle(mMask_mc, 0,0,Options.thumbnailWidth,Options.thumbnailHeight);	
		mMask_mc.endFill();	
		mLoader_mc.setMask(mMask_mc);	
				
		drawFrame();
		
		mFrame_mc._visible = true;
		
		//do popup thumb animation
		mPopupTween = new TweenExtended(mBase_mc,["_x","_y","_xscale","_yscale","_alpha"],Regular.easeOut,[Options.thumbnailWidth/2,Options.thumbnailHeight/2,1,1,0],[0,0,100,100,100],10,false);				
		mClip_mc._visible = true;
		
		mThumbArea.loadNextThumb(mIndex);
	}
	
	/**
	 * Thumb jpg could not be found
	 */
	function onLoadError(){
		
		mInit_mc._visible = false;
		
		mImgLoadStatus_mc.clear();				
		//show error icon
		mError_mc = mBase_mc.attachMovie("ErrorIcon","error",mBase_mc.getNextHighestDepth());			
		mError_mc._x = Options.thumbnailWidth/2;
		mError_mc._y = Options.thumbnailHeight/2;
		mError_mc._width = Options.thumbnailWidth/2;
		mError_mc._height = Options.thumbnailHeight/2;	
		var col = new Color(mError_mc);
		col.setRGB(mXMLManager.thumbnailFrameColor);	
		new Tween(mError_mc,"_alpha",null,0,100,10,false);		
		
		mThumbArea.loadNextThumb(mIndex);
	}
	
	public function showImageProgress(percent:Number){
		
		if (!mThumbLoaded || mImageLoaded) return;			
		mImgLoadStatus_mc.clear();	
		mImgLoadStatus_mc.beginFill(mXMLManager.thumbnailFrameColor,100);
		
		var inset = Options.thumbnailFrameWidth;
		var barWidth = Math.round((Options.thumbnailWidth-2*inset)*percent/100);
		RectUtil.rectangle(mImgLoadStatus_mc,inset,Options.thumbnailHeight-inset-Options.thumbnailLoadBarHeight,barWidth+inset,Options.thumbnailHeight-inset);						
		mImgLoadStatus_mc.endFill();			
	}
	
	public function showImageLoadComplete(){		
		mImageLoaded = true;	
		mImgLoadStatus_mc.clear();
		new Tween(mLoader_mc,"_alpha",null,Options.thumbnailLoadingAlpha,100,10,false);			
		if (mSelected) mViewedIcon_mc._visible = true;
			
	}
	
	private function enableMouseActions(){
		mClip_mc.onRelease = Delegate.create(this,onRelease);
		mClip_mc.onPress = Delegate.create(this,onPress);
		mClip_mc.onRollOver = Delegate.create(this,onRollOver);
		mClip_mc.onRollOut = Delegate.create(this,onRollOut);	
		mClip_mc.onDragOut = Delegate.create(this,onDragOut);
		mClip_mc.useHandCursor = false;	
	}
		
	private function onRelease(){
		if (mSelected ) return;
		mBase_mc._x -= Options.thumbnailClickShift;
		mBase_mc._y -= Options.thumbnailClickShift;
		mThumbArea.selectedThumbIndex =  mIndex;
					
	}
	
	private function onPress(){
		if (mSelected ) return;
		mBase_mc._x += Options.thumbnailClickShift;
		mBase_mc._y += Options.thumbnailClickShift;					
	}
	
	private function onRollOver(){
		if (mSelected ) return;			
		mFrameTwn.continueTo(Options.thumbnailRollOverFrameWidth,sFrameTweenLen);
		if (Options.showCaptionOnRollover) mStageManager.showCaption(mIndex);
	}
	
	private function onRollOut(){
		if (mSelected ) return;
		mFrameTwn.continueTo(Options.thumbnailFrameWidth,sFrameTweenLen);
		if (Options.showCaptionOnRollover) mStageManager.showCaption(mImageArea.currentImageIndex);
	}
	
	private function onDragOut(){
		if (mSelected ) return;
		onRollOut();
		onRelease();				
	}
	
	public function select(){	
		mSelected = true;	
		mImageArea.showImage(mIndex);
		drawFrame();
		if (mImageLoaded) mViewedIcon_mc._visible = true;		
		if (!mImageLoaded) mImageArea.loadImageOutOfSequence(mIndex);				
	}
	
	public function unselect(){
		mSelected = false;
		if (mClip_mc.hitTest(_root._xmouse,_root._ymouse)){	
			drawFrame();	
		}else{
			mFrameTwn.continueTo(Options.thumbnailFrameWidth,sFrameTweenLen);	
		}		
	}

	public static function get thumbWidth():Number{		
		return Options.thumbnailWidth;	
	}
	
	public static function get thumbHeight():Number{		
		return Options.thumbnailHeight;	
	}
		
	public static function get clickShift():Number{		
		return Options.thumbnailClickShift;	
	}
	
	public static function get selectedFrameOffset():Number{		
		return Options.thumbnailSelectedFrameOffset;	
	}
	
	public static function get padding():Number{		
		return Options.thumbnailPadding;	
	}
	
	public function setPosn(x:Number,y:Number):Void{		
		mClip_mc._x = x;
		mClip_mc._y = y;				
	}	
	
	private function drawFrame(){
		mFrame_mc.clear();		
		mFrame_mc.beginFill(mXMLManager.thumbnailFrameColor, 100);		
		if (mSelected){
			RectUtil.hollowRectangle(mFrame_mc,-Options.thumbnailSelectedFrameOffset,-Options.thumbnailSelectedFrameOffset,Options.thumbnailWidth+Options.thumbnailSelectedFrameOffset,Options.thumbnailHeight+Options.thumbnailSelectedFrameOffset,Options.thumbnailRollOverFrameWidth);
		}else{
			RectUtil.hollowRectangle(mFrame_mc,0,0,Options.thumbnailWidth,Options.thumbnailHeight,mFrameWidth);	
		}
		mFrame_mc.endFill();			
	}	
	
}